import json
import uuid
from django.db import models
from django.utils import timezone
import stix2


class IdentityIDField(models.CharField):
    def pre_save(self, model_instance, add):
        if add and not getattr(model_instance, self.attname, None):
            value = "identity--" + str(uuid.uuid4())
            setattr(model_instance, self.attname, value)
            return value
        return super().pre_save(model_instance, add)


class AutoUpdatedModifiedField(models.DateTimeField):
    def pre_save(self, model_instance, add):
        value = getattr(model_instance, self.attname, None)
        if not add:
            value = timezone.now()
            setattr(model_instance, self.attname, value)
        return value


class Identity(models.Model):
    id = IdentityIDField(primary_key=True, max_length=64, default="")
    created = models.DateTimeField(default=None)
    modified = AutoUpdatedModifiedField(default=None)
    stix = models.JSONField(default=dict)

    def save(self, *args, **kwargs) -> None:
        if not self.created:
            self.created = timezone.now()
        if not self.modified:
            self.modified = self.created
        return super().save(*args, **kwargs)

    @property
    def static_dict(self):
        return {
            "type": "identity",
            "spec_version": "2.1",
            "id": self.id,
            "created": self.created,
            "modified": self.modified,
        }

    @property
    def identity(self):
        data = self.static_dict
        data.update(self.stix)
        return stix2.Identity(**data)

    @property
    def dict(self) -> dict:
        return json.loads(self.identity.serialize())



import json
import stix2
import stix2.exceptions
from dogesec_commons.identity.models import Identity
from rest_framework import serializers, validators
from dogesec_commons.identity.models import Identity
from rest_framework import serializers, validators


class IdentitySerializer(serializers.ModelSerializer):
    instance: Identity
    id = serializers.CharField(
        validators=[validators.UniqueValidator(queryset=Identity.objects.all())]
    )

    class Meta:
        model = Identity
        fields = "__all__"
        extra_kwargs = {
            "stix": {"write_only": True, "required": False},
        }

    def to_representation(self, instance):
        return instance.dict

    def to_internal_value(self, data):
        data = dict(data.copy())
        ## do initial validation
        if not self.instance:
            super().to_internal_value(data)
        # actual validation

        if not isinstance(data, dict):
            raise serializers.ValidationError(
                {"non_field_errors": ["STIX Identity must be a JSON object (dict)."]}
            )
        invalid_update_keys = [
            "created",
            "id",
            "modified",
        ]
        if self.instance:
            invalidated_error_message = {}
            for key in invalid_update_keys:
                if key in data and data[key] != self.instance.dict.get(key):
                    invalidated_error_message[key] = [
                        f"Cannot modify '{key}' of an existing Identity."
                    ]

            if invalidated_error_message:
                raise serializers.ValidationError(invalidated_error_message)

        if self.instance:
            # For updates, merge existing stix data with new data
            data.update(self.instance.static_dict)
        try:
            identity = stix2.Identity(**data)
        except stix2.exceptions.STIXError as exc:
            raise serializers.ValidationError(
                {"stix_validation_error": [f"Invalid STIX Identity object: {exc}"]}
            )
        value_dict = json.loads(identity.serialize())
        retval = {}
        retval["stix"] = value_dict.copy()
        retval["id"] = retval["stix"].pop("id")
        retval["modified"] = retval["stix"].pop("modified")
        retval["created"] = retval["stix"].pop("created")
        return retval

    def update(self, instance: Identity, validated_data):
        return super().update(instance, validated_data)

    def get_schema(self):
        UUID_RE = (
            "[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}"
        )
        return {
            "$schema": "http://json-schema.org/draft-07/schema#",
            "$id": "https://example.com/stix/identity.schema.json",
            "title": "STIX 2.1 Identity Object",
            "type": "object",
            "required": [
                "type",
                "spec_version",
                "id",
                "created",
                "modified",
                "name",
                "identity_class",
            ],
            "properties": {
                "type": {"const": "identity"},
                "spec_version": {"const": "2.1"},
                "id": {
                    "type": "string",
                    "pattern": f"^identity--{UUID_RE}$",
                },
                "created": {"type": "string", "format": "date-time"},
                "modified": {"type": "string", "format": "date-time"},
                "created_by_ref": {
                    "type": "string",
                    "pattern": f"^identity--{UUID_RE}$",
                },
                "revoked": {"type": "boolean"},
                "labels": {"type": "array", "items": {"type": "string"}},
                "confidence": {"type": "integer", "minimum": 0, "maximum": 100},
                "lang": {"type": "string"},
                "name": {"type": "string", "minLength": 1},
                "description": {"type": "string"},
                "identity_class": {
                    "type": "string",
                },
                "sectors": {"type": "array", "items": {"type": "string"}},
                "contact_information": {"type": "string"},
                "object_marking_refs": {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": f"^marking-definition--{UUID_RE}$",
                    },
                },
                "external_references": {
                    "type": "array",
                    "items": {
                        "type": "object",
                        "required": ["source_name"],
                        "properties": {
                            "source_name": {"type": "string"},
                            "description": {"type": "string"},
                            "url": {"type": "string", "format": "uri"},
                            "external_id": {"type": "string"},
                        },
                        "additionalProperties": False,
                    },
                },
                "extensions": {
                    "type": "object",
                    "additionalProperties": {"type": "object"},
                },
            },
            "additionalProperties": True,
        }



import uuid
from stix2.utils import format_datetime, parse_into_datetime
import pytest
from dogesec_commons.identity import models, serializers

@pytest.fixture
def identity(db):
    """Create a test identity (feed owner)."""
    identity_s = serializers.IdentitySerializer(
        data=dict(
            id="identity--73faab8f-9a95-4417-a2db-c1a8b73c7029",
            name="Test Identity",
            identity_class="organization",
            sectors=["technology"],
            created="2020-01-01T00:00:00.000Z",
            modified="2020-01-01T00:00:00.000Z",
        )
    )
    identity_s.is_valid(raise_exception=True)
    identity: models.Identity = identity_s.save()
    identity.refresh_from_db()
    yield identity

@pytest.fixture(autouse=True)
def always_use_db(db):
    """Ensure all tests use the database."""
    pass

def test_update_identity_bad_data(client, identity):
    data = {
        "id": "identity--" + str(uuid.uuid4()),
        "name": "Should Fail",
        "created": "2025-10-09T01:01:01.001Z",
    }

    resp = client.put(
        f"/identities/{identity.id}/",
        data,
        content_type="application/json",
    )
    assert resp.status_code == 400
    errors = resp.json()["details"]
    assert errors["id"] == ["Cannot modify 'id' of an existing Identity."]
    assert errors["created"] == ["Cannot modify 'created' of an existing Identity."]


def test_update_identity_good_data(client, identity):
    data = {
        "name": "Updated Name",
        "sectors": ["finance", "technology"],
    }

    resp = client.put(
        f"/identities/{identity.id}/",
        data,
        content_type="application/json",
    )
    assert resp.status_code == 200
    updated_identity = resp.json()
    assert updated_identity["name"] == "Updated Name"
    assert updated_identity["sectors"] == ["finance", "technology"]
    assert updated_identity["id"] == identity.id
    assert parse_into_datetime(updated_identity["created"]) == identity.created


def test_create_identity(client):
    data = {
        "name": "New Identity",
        "identity_class": "individual",
        "sectors": ["healthcare"],
        "id": "identity--" + str(uuid.uuid4()),
    }

    resp = client.post(
        "/identities/",
        data,
        content_type="application/json",
    )
    assert resp.status_code == 201, resp.content
    new_identity = resp.json()
    assert new_identity["name"] == "New Identity"
    assert new_identity["identity_class"] == "individual"
    assert new_identity["sectors"] == ["healthcare"]
    assert new_identity["id"] == data["id"]
    assert new_identity["created"] == new_identity["modified"]


def test_create_identity_with_modified(client):
    data = {
        "name": "New Identity",
        "identity_class": "individual",
        "sectors": ["healthcare"],
        "id": "identity--" + str(uuid.uuid4()),
        "created": "2020-01-01T00:00:00.000Z",
        "modified": "2023-01-01T00:00:00.000Z",
    }

    resp = client.post(
        "/identities/",
        data,
        content_type="application/json",
    )
    assert resp.status_code == 201, resp.content
    new_identity = resp.json()
    assert new_identity["name"] == "New Identity"
    assert new_identity["identity_class"] == "individual"
    assert new_identity["sectors"] == ["healthcare"]
    assert new_identity["id"] == data["id"]
    assert new_identity["created"] == data["created"]
    assert new_identity["modified"] == data["modified"]


def test_put_identity_updates_modified(client, identity):
    payload = {
        "name": "Updated Identity",
        "identity_class": "individual",
        "contact_information": "updated-email@dogesec.com",
    }
    resp = client.put(
        f"/identities/{identity.id}/",
        payload,
        content_type="application/json",
    )
    assert resp.status_code == 200, resp.content
    data = resp.json()
    assert (
        data["modified"] != data["created"]
    ), "modified timestamp should be updated on put"
    assert "sectors" not in data, "sectors should be removed if not in payload"
    assert data == {
        "type": "identity",
        "spec_version": "2.1",
        "id": "identity--73faab8f-9a95-4417-a2db-c1a8b73c7029",
        "created": "2020-01-01T00:00:00.000Z",
        "modified": data["modified"],
        "name": "Updated Identity",
        "identity_class": "individual",
        "contact_information": "updated-email@dogesec.com",
    }


def test_create_identity_unique_id(client, identity):
    data = {
        "id": identity.id,
        "name": "Duplicate ID Identity",
        "created": "2020-01-01T00:00:00.000Z",
        "modified": "2020-01-01T00:00:00.000Z",
        "identity_class": "individual",
        "sectors": ["education"],
    }

    resp = client.post(
        "/identities/",
        data,
        content_type="application/json",
    )
    assert resp.status_code == 400
    errors = resp.json()["details"]
    assert errors["id"] == ["This field must be unique."]



"""
Views for the Cyber Threat Exchange server.
"""

SEMANTIC_SEARCH_SORT_FIELDS = [
    "modified_descending",
    "modified_ascending",
    "created_ascending",
    "created_descending",
    "name_ascending",
    "name_descending",
    "type_ascending",
    "type_descending",
]

from django_filters.rest_framework import (
    CharFilter,
    DjangoFilterBackend,
    FilterSet,
    FilterSet,
    DjangoFilterBackend,
    CharFilter,
)
from drf_spectacular.types import OpenApiTypes
from drf_spectacular.utils import extend_schema, extend_schema_view, OpenApiParameter
from rest_framework import viewsets
from dogesec_commons.utils import Ordering, Pagination
from dogesec_commons.utils.schemas import DEFAULT_400_RESPONSE
from dogesec_commons.identity import serializers, models


@extend_schema_view(
    list=extend_schema(
        summary="List Identities",
        description="List all STIX Identity objects that can be used to create feeds.",
    ),
    retrieve=extend_schema(
        summary="Retrieve an Identity",
        description="Retrieve a STIX Identity object by its ID.",
    ),
    create=extend_schema(
        summary="Create an Identity",
        description="Create a new STIX Identity object.",
        responses={201: serializers.IdentitySerializer, 400: DEFAULT_400_RESPONSE},
    ),
    update=extend_schema(
        summary="Update an Identity",
        description="Update an existing STIX Identity object.",
        responses={200: serializers.IdentitySerializer, 400: DEFAULT_400_RESPONSE},
    ),
    destroy=extend_schema(
        summary="Delete an Identity",
        description="Delete a STIX Identity object.",
    ),
)
class IdentityView(viewsets.ModelViewSet):  # Changed from ReadOnlyModelViewSet
    """
    A viewset for viewing Identities.
    Identities are the owners of feeds.
    """

    http_method_names = ["get", "post", "put", "delete"]
    openapi_tags = ["Identities"]
    queryset = models.Identity.objects.all()
    serializer_class = serializers.IdentitySerializer
    pagination_class = Pagination("identities")
    lookup_field = "id"
    lookup_url_kwarg = "identity_id"
    lookup_value_regex = (
        r"identity--[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}"
    )
    filter_backends = [DjangoFilterBackend, Ordering]
    ordering_fields = ["created", "modified"]
    ordering = "modified_descending"
    openapi_path_params = [
        OpenApiParameter(
            "identity_id",
            type=OpenApiTypes.STR,
            location=OpenApiParameter.PATH,
            description="The ID of the Identity object (e.g. `identity--643fea2b-5da6-47a9-9433-f8e97669f75b`)",
        )
    ]

    class filterset_class(FilterSet):
        name = CharFilter(
            field_name="stix__name",
            lookup_expr="icontains",
            help_text="Filter by identity name (case-insensitive, partial match). e.g. `oge` would match `dogesec`, `DOGESEC`, etc.",
        )



import time
from urllib.parse import urlencode
import schemathesis
import pytest
from schemathesis.core.transport import Response as SchemathesisResponse
from dogesec_commons.identity.serializers import IdentitySerializer
from dogesec_commons.wsgi import application as wsgi_app
from rest_framework.response import Response as DRFResponse
from schemathesis.specs.openapi.checks import (
    negative_data_rejection,
    positive_data_acceptance,
)
from schemathesis.config import GenerationConfig
from hypothesis import strategies

schema = schemathesis.openapi.from_wsgi(
    "/api/schema/?format=json",
    wsgi_app,
)
schema.config.base_url = "http://localhost:8005/"
schema.config.generation = GenerationConfig(allow_x00=False)


@pytest.fixture(autouse=True)
def identities(db):
    identities = []
    for name, stix_id in [
        ("Test Identity 1", "identity--73faab8f-9a95-4417-a2db-c1a8b73c7029"),
        ("Test Identity 2", "identity--90b66c75-8439-4fe2-8a0b-f2e6ff0d00b6"),
    ]:
        identity_s = IdentitySerializer(
            data=dict(
                id=stix_id,
                name=name,
                identity_class="organization",
                sectors=["technology"],
                created="2020-01-01T00:00:00.000Z",
                modified="2020-01-01T00:00:00.000Z",
            )
        )
        identity_s.is_valid(raise_exception=True)
        identity = identity_s.save()
        identity.refresh_from_db()
        identities.append(identity)
    yield identities


@pytest.fixture(autouse=True)
def override_transport(monkeypatch, client):
    from schemathesis.transport.wsgi import WSGI_TRANSPORT, WSGITransport

    class Transport(WSGITransport):
        def __init__(self):
            super().__init__()
            self._copy_serializers_from(WSGI_TRANSPORT)

        @staticmethod
        def case_as_request(case):
            from schemathesis.transport.requests import REQUESTS_TRANSPORT
            import requests

            r_dict = REQUESTS_TRANSPORT.serialize_case(
                case,
                base_url=case.operation.base_url,
            )
            return requests.Request(**r_dict).prepare()

        def send(self, case: schemathesis.Case, *args, **kwargs):
            t = time.time()
            case.headers.pop("Authorization", "")
            serialized_request = WSGI_TRANSPORT.serialize_case(case)
            serialized_request.update(
                QUERY_STRING=urlencode(serialized_request["query_string"])
            )
            response: DRFResponse = client.generic(**serialized_request)
            elapsed = time.time() - t
            return SchemathesisResponse(
                response.status_code,
                headers={k: [v] for k, v in response.headers.items()},
                content=response.content,
                request=self.case_as_request(case),
                elapsed=elapsed,
                verify=True,
            )

    ## patch transport.get
    from schemathesis import transport

    monkeypatch.setattr(transport, "get", lambda _: Transport())


@pytest.mark.django_db(transaction=True)
@schema.given(
    identity_id=strategies.sampled_from(
        [
            "identity--73faab8f-9a95-4417-a2db-c1a8b73c7029",
            "identity--90b66c75-8439-4fe2-8a0b-f2e6ff0d00b6",
            "identity--89718333-15ec-4ffd-945e-831fe133f58c",
        ]
    )
)
@schema.parametrize()
def test_api(case: schemathesis.Case, **kwargs):
    for k, v in kwargs.items():
        if k in case.path_parameters:
            case.path_parameters[k] = v
    case.call_and_validate(
        excluded_checks=[negative_data_rejection, positive_data_acceptance]
    )



# Generated by Django 5.2.4 on 2026-01-05 12:10

import dogesec_commons.identity.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Identity',
            fields=[
                ('id', dogesec_commons.identity.models.IdentityIDField(default='', max_length=64, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(default=None)),
                ('modified', dogesec_commons.identity.models.AutoUpdatedModifiedField(default=None)),
                ('stix', models.JSONField(default=dict)),
            ],
        ),
    ]
